#!/usr/bin/python
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
import string
import time
from xml.dom.minidom import getDOMImplementation, parse
import xml.dom.ext
from tags import Tags, Attributes, Constants
import version

class DocumentWrapper:

    def __init__(self):
        self.document = None
        self.Debugging = False
        
        
    def findOrCreateElement(self, parent, elementName):
        try:
            child = parent.getElementsByTagName(elementName)[0]
        except IndexError: 
           child = self.document.createElement(elementName)
           parent.appendChild(child)
        return child
    
    def setTextNode(self, element, value):
        value = self.removeBadCharacters(value)
        # is there an existing child text node?
        textNode = self.getTextNode(element)
        if textNode:
            textNode.data = value
            return
        
        # else no existing text child note, create it
        textNode = self.document.createTextNode(value)
        element.appendChild(textNode)  
        
    def getTextNode(self, element):
        for node in element.childNodes:
            if node.nodeType == node.TEXT_NODE:
                return node
        return None
         
    def timeToString(self, theTime):
        return time.strftime(Constants.DATETIMEFORMAT, theTime)

    def removeBadCharacters(self, value):
        badnumbers = [0,8,27,246]
        badchars = "".join(chr(i) for i in badnumbers) + "|"
        chars = "".join(chr(i) for i in range(256))
        return value.translate(chars, badchars)

class Certification(DocumentWrapper):

    def __init__(self, debugging=False):
        self.currentTestElement = None
        self.Debugging = debugging
    
    def new(self):
       dom = getDOMImplementation()
       self.document = dom.createDocument(None, Tags.certification_test, None)
       topElement = self.document.documentElement
       topElement.setAttribute(Attributes.hts_version, version.version)
       topElement.setAttribute(Attributes.hts_release, version.release)
       # text = newdoc.createTextNode('Some textual content.')
       # top_element.appendChild(text)
       
       
    def load(self, filename):
        file = open(filename, "r")
        self.document = parse(file)
        file.close()
        
    def save(self, filename):
        file = open(filename, "w")
        xml.dom.ext.PrettyPrint(self.document, file)
        file.close()
   
    
    def getDiscoverTime(self):
        return self.document.documentElement.getAttribute(Attributes.discover_time)
    
    def setDiscoverTime(self, discoverTime):
        self.document.documentElement.setAttribute(Attributes.discover_time, self.timeToString(discoverTime))
        
    def getPlanTime(self):
        return self.document.documentElement.getAttribute(Attributes.plan_time)

    def setPlanTime(self, planTime):
        self.document.documentElement.setAttribute(Attributes.plan_time, self.timeToString(planTime ))
        
    def addTestRun(self):
        numberOfRuns = self.getNumberOfTestRuns() + 1
        self.document.documentElement.setAttribute(Attributes.number_of_runs, "%u" % numberOfRuns)
        
    def getNumberOfTestRuns(self):
        numberOfRunsString = self.document.documentElement.getAttribute(Attributes.number_of_runs)
        if numberOfRunsString:
            return int(numberOfRunsString)
        else:
            return 0
        
    def setTestServer(self, testServer):
        if testServer:
            self.setHardware(Tags.test_server, testServer)
            
    def getTestServer(self):
        return self.getHardware(Tags.test_server)
        
        
    def setHardware(self, tag, value):
        hardwareElement = self.findOrCreateElement(self.document.documentElement, Tags.hardware)
        element = self.findOrCreateElement(hardwareElement, tag)
        self.setTextNode(element, value)
    
    def getHardware(self, tag):
        hardwareElement = self.findOrCreateElement(self.document.documentElement, Tags.hardware)
        element = self.findOrCreateElement(hardwareElement, tag)
        textNode = self.getTextNode(element)
        if textNode: 
            return textNode.data
        # otherwise
        return ""
             
    def setOS(self, tag, value):
        certificationElement = self.findOrCreateElement(self.document.documentElement, Tags.certification)
        osElement = self.findOrCreateElement(certificationElement, Tags.os)
        element = self.findOrCreateElement(osElement, tag)
        self.setTextNode(element, value)
        
    def getOS(self, tag):
        certificationElement = self.findOrCreateElement(self.document.documentElement, Tags.certification)
        osElement = self.findOrCreateElement(certificationElement, Tags.os)
        element = self.findOrCreateElement(osElement, tag)
        return self.getTextNode(element).data
        
        
    def appendTest(self, name, udi, label):
        if self.Debugging:
            print "Certification.appendTest(%s, %s, %s)" % (name, udi, label)
        testElement = self.document.createElement(Tags.test)
        testElement.setAttribute(Attributes.name, name)
        testElement.setAttribute(Attributes.udi, udi)
        testElement.setAttribute(Attributes.label, label)
        self.document.documentElement.appendChild(testElement)
   
    def getInfoTest(self):                                                        #YK
        testElements = self.document.getElementsByTagName(Tags.test)              #YK
        infotest = list()                                                         #YK
        for testElement in testElements:                                          #YK
            testName = testElement.getAttribute(Attributes.name)                  #YK
            if testName == "info":                                                #YK
               infotest = Test(testElement, self.document)                        #YK
        return infotest                                                           #YK
 
    def getTests(self, name="", onlyFailures=False):
        testElements = self.document.getElementsByTagName(Tags.test)
        tests = list()
        for testElement in testElements:
            testName = testElement.getAttribute(Attributes.name)
            if name == "" or string.find(testName, name) >= 0:
                test = Test(testElement, self.document)
                if not onlyFailures or (onlyFailures and test.hasFailures()):
                    tests.append(test)
        return tests
    
    def getTestsGroupedByName(self):
        testElements = self.document.getElementsByTagName(Tags.test)
        testsByName = dict()
        for testElement in testElements:
            test = Test(testElement, self.document)
            testName = testElement.getAttribute(Attributes.name)
            if testName not in testsByName:
                testsByName[testName] =  list()
            testsByName[testName].append(test)
        return testsByName
    
    def getTestsByUDI(self, udi):
        testElements = self.document.getElementsByTagName(Tags.test)
        tests = list()
        for testElement in testElements:
            testUDI = testElement.getAttribute(Attributes.udi)
            if string.find(testUDI, udi) >= 0:
                tests.append(Test(testElement, self.document))
        return tests
    
    def getVersion(self):
        return self.document.documentElement.getAttribute(Attributes.hts_version)
    def getRelease(self):
        return self.document.documentElement.getAttribute(Attributes.hts_release)
    
    def getTest(self, testName):
        testElements = self.document.documentElement.getElementsByTagName(Tags.test)
        for testElement in testElements:
            if testElement.getAttribute(Attributes.name) == testName:
                return Test(testElement)         
        return None
    
    def getTestCounts(self, runNumber):
        testElements = self.document.documentElement.getElementsByTagName(Tags.test)
        count = 0
        runCount = 0
        passed = 0
        failed = 0
        start = None
        for testElement in testElements:
            test = Test(testElement, self.document)      
            
            count = count + 1
            if test.wasRun(runNumber):
                run = test.getRun(runNumber)   
                runCount = runCount + 1
                if run.passed():
                    passed = passed + 1
                else:
                    failed = failed + 1
                if not start:
                    time = run.getRunTime()
                    if time:
                        start = time
                
                
        return {'count':count,'run':runCount, 'passed':passed, 'failed':failed, 'start':start}
    
    def getCombinedTestCounts(self):
        testElements = self.document.documentElement.getElementsByTagName(Tags.test)
        count = 0
        runCount = 0
        neverPassedCount = 0
        neverFailedCount = 0
        start = None
        for testElement in testElements:
            test = Test(testElement, self.document)               
            count = count + 1
            wasRun = False
            passed = False
            failed = False
            for run in test.getRuns():
                if run.wasRun():
                    wasRun = True
                    if run.passed():
                        passed = True
                    else:
                        failed = True
                    
                    if not start:
                        time = run.getRunTime()
                        if time:
                            start = time
            if wasRun: 
                runCount = runCount +1
                if not passed: neverPassedCount = neverPassedCount +1
                if not failed: neverFailedCount = neverFailedCount +1
           
                
                
        return {'count':count,'run':runCount, 'neverPassed':neverPassedCount, 'neverFailed':neverFailedCount, 'start':start}
    
    def getName(self):
        # name is: <vendor>-<model>-<os><release>_<arch>
       name =  "%s-%s-%s%s_%s" % (self.getHardware(Tags.vendor), self.getHardware(Tags.model), self.getOS(Tags.name), self.getOS(Tags.release), self.getHardware(Tags.arch))
       return name.replace(" ", "_")  
    
    
class Test(DocumentWrapper):

    def __init__(self, testElement, document):
        self.testElement = testElement
        self.document = document
    
    def getName(self):
        return self.testElement.getAttribute(Attributes.name)
    
    def getUDI(self):
        return self.testElement.getAttribute(Attributes.udi)
    
    def getLabel(self):
        return self.testElement.getAttribute(Attributes.label)
    
    def isDisabled(self):
        # print "attribute: %s" % self.testElement.getAttribute(Attributes.disabled)
        return (Constants.TRUE in self.testElement.getAttribute(Attributes.disabled))
    
    def setDisabled(self, disabled):
        if disabled:
            value = Constants.TRUE
        else:
            value = Constants.FALSE
        self.testElement.setAttribute(Attributes.disabled, value)
    
    
    def newTestRun(self):
        runElement = self.document.createElement(Tags.run)
        self.testElement.appendChild(runElement)
        run = Run(runElement, self.document)
        return run
    
    def getRuns(self, number=-1, latest=False):
        runElements = self.testElement.getElementsByTagName(Tags.run)
        runs = list()
        lastRun = None
        for runElement in runElements:
            run = Run(runElement, self.document)
            if number==-1 or number == run.getNumber():                
                if not latest:
                    runs.append(run)
                else: 
                    lastRun = run
                        
        if latest and lastRun:
            runs.append(lastTest)
        return runs
    
    def getRun(self, number):
        runs = self.getRuns(number)
        try:
            return runs[0]
        except IndexError: 
            pass
        return None
        
    
    def wasRun(self, runNumber):
        run = self.getRun(runNumber)
        if run:
            return run.wasRun();
         
        #otherwise
        return False
    
    def hasFailures(self):
        for run in self.getRuns():
            if run.wasRun() and run.getSummary() == Constants.FAIL:
                return True
        return False
    
   
            
class Run(DocumentWrapper):

    def __init__(self, runElement, document):
        self.runElement = runElement
        self.document = document
    
    def getNumber(self):
        return int(self.runElement.getAttribute(Attributes.number))
    
    def setNumber(self, number):
        self.runElement.setAttribute(Attributes.number, "%u" % number)
        
    def getMode(self):
        return self.runElement.getAttribute(Attributes.mode)
    
    def setMode(self, mode):
        self.runElement.setAttribute(Attributes.mode, mode)
    
    def getRunTime(self):
        return self.runElement.getAttribute(Attributes.run_time)

    def setRunTime(self, runTime):
        self.runElement.setAttribute(Attributes.run_time, self.timeToString(runTime ))
    
    def getReturnValue(self):
        return self.runElement.getAttribute(Attributes.return_value)
    
    def setReturnValue(self, returnValue):
        self.runElement.setAttribute(Attributes.return_value, ("%u") % returnValue)
        
    def getSummary(self):
        try:
            summaryElement = self.runElement.getElementsByTagName(Tags.summary)[0]
            return summaryElement.childNodes[0].data
        except IndexError:
            return None
        
    def setSummary(self, summary):
        summaryElement = self.findOrCreateElement(self.runElement, Tags.summary)
        try:
            summaryElement.childNodes[0].data = summary
        except IndexError:
            textNode = self.document.createTextNode(summary)
            summaryElement.appendChild(textNode)
    
    def wasRun(self):
        try:
            system = self.runElement.getElementsByTagName(Tags.system)[0]
        except IndexError: 
           return False
        return True
    
    def passed(self):
        if not self.wasRun():
            return False
        return self.getSummary() == "PASS"
        
    
    def setHalLog(self, log):
        halElement = self.findOrCreateElement(self.runElement, Tags.hal)
        textNode = self.document.createTextNode(log)
        halElement.appendChild(textNode)
        
    def getHalLog(self):
        try:
            halElement = self.runElement.getElementsByTagName(Tags.hal)[0]
            return halElement.childNodes[0].data
        except IndexError:
            return None  
    
    def getSystemLog(self):
        try:
            systemElement = self.runElement.getElementsByTagName(Tags.system)[0]
            return systemElement.childNodes[0].data
        except IndexError:
            return None  
        
    def setSystemLog(self, log):
        systemElement = self.findOrCreateElement(self.runElement, Tags.system)
        textNode = self.document.createTextNode(log)
        systemElement.appendChild(textNode)
    
    def getTestOutput(self):
        try:
            outputElement = self.runElement.getElementsByTagName(Tags.output)[0]
            return outputElement.childNodes[0].data
        except IndexError:
            return None  
        
    def setTestOutput(self, output):
        outputElement = self.findOrCreateElement(self.runElement, Tags.output)
        textNode = self.setTextNode(outputElement, output)
        
    def getResults(self, resultsFile):
        try:
            file = open(resultsFile, "r")
        except IOError: # no results file?
            self.setSummary(Constants.ABORT)
            return
        
        resultDocument = parse(resultsFile)
        file.close()
        resultsElement = resultDocument.documentElement
        name = resultsElement.getAttribute(Attributes.name)
        # should we check if name == self.getName():
        # metric =  topElement.getAttribute(Attributes.metric)
        summaryElement = resultsElement.getElementsByTagName(Tags.summary)[0]
        result = summaryElement.childNodes[0].data
        self.setSummary(result)          


if __name__ == "__main__":
    test = Certification()
    test.new()
    file=open("test.xml","w")
    test.save(file)
       

