#!/usr/bin/python
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#

import os
import sys
import syslog
import errno
import tempfile
import md5
import time
import copy
from dbus import dbus_bindings
from hts.controller import Controller
from hts.tags import Tags, Attributes, Constants
from hts.certificationtest import Certification, Test, Run  
from hts.report import Report 
import hts.version

from hts.hal import Hal
from hts.testinfo import TestInfo

    
class HardwareTestHarness(Controller):
    
    LogDirectory="/var/log/hts"
    Debugging=True
    
    def __init__(self, options):
        self.options = options
        self.dataDirectory = options.data
        self.testDirectory = options.source
        self.configFile = "%s/%s" % (options.data, options.cfg)
        self.planFile = "%s/%s" % (options.data, options.plan)
        self.resultsFile = "%s/%s" % (options.data, options.results)
        self.testServer = options.server
        self.Debugging = options.debug
        self.certification = Certification(self.Debugging)
        self.systemLogMarker = "hts/runtests[%s]" % os.getpid()
        self.runMode = options.mode
        
    def do(self, args):
        self.makeDirectoryPath(self.dataDirectory)
        command = args[0]
        if (command == 'discover'):
            return self.doDiscover()
        elif (command == 'plan'):           
            return self.doPlan()
        elif (command == 'certify'):           
            return self.doCertify()
        elif (command == 'print'):
            if self.options.report in (Constants.full, Constants.summary):
                try:
                    self.certification.load(self.resultsFile)
                    return self.doPrint()
                except IOError:
                    pass
            #otherwise
            try:
                self.certification.load(self.planFile)
                return self.doPrint()
            except IOError:
                pass
            #otherwise
            print ("No test plan or results to print.  Try running \"hts plan --data %s\" first." % self.dataDirectory)
            return
            
        elif (command == 'submit'):
            self.certification.load(self.resultsFile)
            return self.doSubmit()      
        elif (command == 'run'):
            return self.doRun()
        elif (command == 'server'):
            return self.doServer(args)
        elif (command == 'version'):
            print "HTS version %s, release %s" % (hts.version.version, hts.version.release)
            return 0
        else:
            print "unknown command: " + command
                

    def doDiscover(self):        
        self.certification.new()
        if self.testServer:
            self.certification.setTestServer(self.testServer)
        self.getBiosInfo()
        print "Hardware: %s %s %s" % (self.certification.getHardware(Tags.vendor), 
                                      self.certification.getHardware(Tags.make), 
                                      self.certification.getHardware(Tags.model))
        self.certification.setOS(Tags.name, self.getReleaseName())
        releaseNumber = self.getReleaseNumber()
        if releaseNumber:
            self.certification.setOS(Tags.release, releaseNumber)
        else:
            print "Error: could not determine OS release number"
            return False      
        print "OS: %s %s" % (self.certification.getOS(Tags.name), self.certification.getOS(Tags.release))
        self.certification.setDiscoverTime(self.getCurrentUTCTime())
        self.certification.save(self.configFile)
        print "saved configuration to %s" % self.configFile
        return True
 
    def doPlan(self):
        # load the config
        try:
            config = Certification(self.Debugging)
            config.load(self.configFile)
            print "loading configuration %s" % self.configFile
        except IOError:
            self.doDiscover()
            config = self.certification
           # print "Error: could not load %s.  Please run: hts discover --data %s" % (self.configFile, self.dataDirectory)
           # return
        
        # try and load an existing plan
        newPlan = True
         # edit an existing plan?       
        editPlan = (self.options.test or self.options.udi) and (self.options.disable or self.options.enable or self.options.add)

        try:
            self.certification.load(self.planFile)
            print "loading test plan %s" % self.planFile
            # check timestamps: did the plan come from the config?
            if self.certification.getDiscoverTime() != config.getDiscoverTime():
                print "Error: Test plan %s is inconsistent with configuration %s" % (self.planFile, self.configFile)
                return
            # otherwise
            newPlan = False
        except IOError:
            # no plan, make a new one from the config
            self.certification = config
               
                
        if newPlan:
            hal=Hal()      
            testInfo=TestInfo(self.testDirectory, self.Debugging)
            testCount = 0
            tests = testInfo.getGeneralTests();
            print "planning device-independant tests..."
            for test in testInfo.getGeneralTests():
                test.setPlanUDI(Constants.GENERAL)
                test.setPlanDeviceLabel(Constants.GENERAL)
            
            print "planning device tests..."    
            for dev in hal.devices():
                for test in testInfo.getTests(dev):
                    deviceLabel = ""
                    testCount = testCount + 1
                    try:
                        deviceLabel = dev['info.product']
                    except KeyError:
                        pass
                    if self.Debugging:
                        print "added test: %s for device: %s (%s)" % (test, dev['info.udi'], deviceLabel)
                    test = copy.copy(test)
                    test.setPlanUDI(dev['info.udi'])
                    test.setPlanDeviceLabel(deviceLabel)
                    tests.append(test)
                    
            # sort by TestMetaData comparison: interactive 1st, then by priority
            tests.sort()
            
            for test in tests:
                self.certification.appendTest(name=test.Name(), udi=test.getPlanUDI(), label=test.getPlanDeviceLabel())
             
            self.certification.setPlanTime(self.getCurrentUTCTime()) 
            print "Created a new plan with %u tests on %u devices" % (len(tests), len(hal.devices())) 
            
        
                                                                       
        if editPlan:
            if self.options.disable or self.options.enable:
                self.setTestsEnabled()
                
            elif self.options.add:
                self.addTest()
                
            
        if self.testServer:
            self.certification.setTestServer(self.testServer)
       
        self.certification.save(self.planFile)
        print "saved test plan to %s" % self.planFile
        
    def doServer(self, args):
        if len(args) > 1:
            subcommand = args[1]
        else:
            subcommand = "start"
        testInfo=TestInfo(self.testDirectory, self.Debugging)
        testCount = 0
        tests = testInfo.getServerModeTests();
        for test in tests:
            self.runServer(test, subcommand)
        
    def doCertify(self):
        try:
            plan = Certification()
            plan.load(self.planFile)
        except IOError:
            print "Test plan %s not found, Please run \"hts plan\" to create a test plan" % self.planFile
            return
        
        try:
            self.certification.load(self.resultsFile)
            print "loaded test results %s" % self.resultsFile
            if plan.getPlanTime() != self.certification.getPlanTime():
                print "Error: the results file %s inconsistent with the test plan %s" % (self.resultsFile, self.planFile)
                return
        except IOError:
            # no results yet, use plan to make the results
            self.certification = plan
                
        if self.options.udi:
            tests = self.certification.getTestsByUDI(self.options.udi)
            try:
                print "running only test %s on device %s" % (tests[0].getName(), self.options.udi)
            except IndexError:
                print "no such test on device %s" % self.options.udi
                return
        elif self.options.test:
            tests = self.certification.getTests(self.options.test)
            try:
                print "running only test %s " % tests[0].getName()
            except IndexError:
                print "no such test %s" % self.options.test
                return
        
        else:
            tests = self.certification.getTests()

        # append info test if it's not there             #YK
        if not self.options.noinfo:
            infotest = self.certification.getInfoTest()      #YK
            INFO_PLANNED = "NO"                              #YK
            for test in tests:                               #YK
                if test.getName() == "info":                 #YK
                    print "info test already been planned"   #YK
                    INFO_PLANNED = "YES"                     #YK
            if INFO_PLANNED == "NO":                         #YK
                tests.append(infotest)                       #YK
            
        # determine test server
        if self.testServer:
            self.certification.setTestServer(self.testServer)
        else:
             if self.certification.getTestServer():
                 self.testServer = self.certification.getTestServer()
            
        hal=Hal()
        # Set up our system logging stuff
        syslog.openlog(self.systemLogMarker)
        syslog.syslog("Beginning test run.")
        # Make a log dir for this testrun
        runDirectory=self.makeTestRunDirectory()
        
        # Start testin'!
        noTestsHaveRun = True
        for test in tests:
            if test.isDisabled():
                print "skipping %s - disabled" % test.getName()
                continue
            
            print ("running %s on %s") % (test.getName(),test.getUDI())
            run = test.newTestRun()
            if noTestsHaveRun:
                noTestsHaveRun = False
                self.certification.addTestRun()
                
            run.setRunTime(self.getCurrentUTCTime())
            run.setMode(self.runMode)
            run.setNumber(self.certification.getNumberOfTestRuns())
            
            # get HAL info
            if test.getUDI() != Constants.GENERAL:
                try:
                    run.setHalLog(hal.dump(test.getUDI()))
                except dbus_bindings.DBusException, e:
                    print "Test error: %s" % e
                    syslog.syslog("Test error: %s" % e)
                    testerror = True
                    continue
                    
            
            # create log dir
            logDirectory=self.makeLogDirectory(runDirectory,test)
            # TODO: RHR2-compat: add tests/ links
            outputFilePath=logDirectory+"/output.log" # set logfile name
            
            # create temp dir
            tmpDirectory=tempfile.mkdtemp('',"hts-"+test.getName()+"-")
            # put test files in temp dir
            self.installTest(self.testDirectory+"/"+test.getName(), tmpDirectory)
        
            # Here goes! Actually run the test.
            syslog.syslog("%s: begin" % test.getName())
            testError = False
            # Walk the tmpdir and find any directory that contains a runtest.sh
            returnValue = 1
            for directory in [i[0] for i in os.walk(tmpDirectory) if 'runtest.sh' in i[2]]:
                try:
                    returnValue = self.runTest(directory, test.getUDI(), outputFilePath)
                    run.getResults(directory+"/test-result.xml")
                except OSError, e:
                    print _("Test error: %s") % e
                    syslog.syslog(_("Test error: %s") % e)
                    testerror = True
        
            # Log the end of the test.
            syslog.syslog("%s: end" % test.getName())
            # If the test did not run correctly, don't record the results.
            if testError: 
                self.removeDirectory(logDirectory)
                continue
            
            # Save the system log
            self.saveSystemLog(test, run)
            # encode return value into output.log
            self.saveOutput(run, returnValue, outputFilePath)
            # print result
            print ("Return value was %u") % returnValue
            # TODO: clean up tmpdir here?
          
        syslog.syslog("Testing complete.")
        syslog.closelog()
        self.certification.save(self.resultsFile)
         
    def doRun(self):
         self.certification.new()
         self.doDiscover()
         self.doPlan()
         self.doCertify()
         
    def doPrint(self):
        report = Report(self.options, self.certification)
        report.Write()            
        
    def doSubmit(self):
        report = Report(self.options, self.certification)
        report.WriteResultsRPM(self.LogDirectory)
       
       
    def getBiosInfo(self):
        arch = self.getArch()
        self.certification.setHardware(Tags.arch, arch)
        if self.Debugging:
            print "Getting Bios Info for %s" % arch
        foundVendor = False
        foundModel = False
        if arch in ['i386','i586','i686','x86_64','ia64']:
            dmidecode = os.popen("/usr/sbin/dmidecode") 
           
            for line in dmidecode:
                if line.find(':') == -1: 
                    continue
                (key,val)=[t.strip() for t in line.split(':',1) if line.find(':')]
                if key == "Vendor": 
                    self.certification.setHardware(Tags.vendor, val)
                    if self.Debugging: print "Vendor: %s" % val
                    foundVendor = True
                if key == "Product Name": 
                    self.certification.setHardware(Tags.model, val)
                    if self.Debugging: print "Model: %s" % val
                    foundModel = True
                if foundModel and foundVendor: 
                    break
            dmidecode.close()
        elif os.path.exists('/proc/device-tree/model'):
            f=open('/proc/device-tree/model')
            model = f.readline().strip()
            if self.Debugging: print "Model: %s" % model
            self.certification.setHardware(Tags.model, model)
            foundModel = True
            f.close()
        elif arch in ['s390','s390x']:
            self.certification.setHardware(Tags.vendor, 'IBM')
            if self.Debugging: print "Vendor: IBM"
            self.certification.setHardware(Tags.make, 'eServer zSeries')
            if self.Debugging: print "Make: eServer zSeries" 
        
        if not (foundModel or foundVendor) and os.system("hal-device | fgrep -q xen") == 0:
            if self.Debugging: print "Running in a PV Guest"
            self.certification.setHardware(Tags.vendor, "Red Hat")
            self.certification.setHardware(Tags.model, "ParaVirt Guest")

    def getArch(self):
        system,node,release,version,machine = os.uname()
        return machine
 
    def getRedHatRelease(self):
        f=open("/etc/redhat-release")
        line=f.readline()
        f.close()
        return line
    
    def getReleaseNumber(self):
        """Return the RHEL release number"""
        line=self.getRedHatRelease()
        # the number is the next string after "release"
        lastword=""
        for word in line.split():
            if lastword == "release":
                return word
            lastword=word
        # otherwise, not found
        return None
    
    def getReleaseName(self):
        """Return the name of the RHEL release running on the system"""
        line=self.getRedHatRelease()
        return line[line.rindex('(')+1:line.rindex(')')]

    def makeTestRunDirectory(self):
        """Create a directory for the current test run"""
        # run number is +1, as test has not yet run
        runNumber = self.certification.getNumberOfTestRuns() + 1
        testRunDirectory=self.LogDirectory + "/runs/%u" % runNumber
        self.removeDirectory(testRunDirectory)
        self.makeDirectoryPath(testRunDirectory)
        return testRunDirectory
    
    def makeLogDirectory(self, directory, test):
        "Create a directory for logs for the currently-running test"
        logdir=directory+"/"+test.getName()
        self.removeDirectory(logdir) # clean it, if it's there
        self.makeDirectoryPath(logdir)
        return logdir
    
    def installTest(self, testDirectory,tmpDirectory):
        "Install a test's files to the named tmpdir for running"
        cwd=os.getcwd()
        os.chdir(testDirectory)
        os.system("make install DEST="+tmpDirectory)
        os.chdir(cwd)
    
    def runTest(self, testDirectory, udi, outputFilePath):
        "Run a test, sending output to the logfile and stdout"
        if self.Debugging:
            print "HardwareTestHarness.runTest:"
            print "    test directory %s  udi %s outputFile %s " % (testDirectory, udi, outputFilePath)
        cwd=os.getcwd()
        os.chdir(testDirectory)
        hal=Hal()
        runCommand = "make %s=%s %s=%s %s=%s " % (
                                 Constants.OUTPUTFILE, outputFilePath, 
                                 Constants.RUNMODE, self.runMode, 
                                 Constants.UDI, udi
                                 )
        if self.testServer:
            runCommand = runCommand + "%s=%s " % (
                                  Constants.TESTSERVER, self.testServer,
                                 )
        runCommand = runCommand + "run"
        print runCommand
        rv = os.WEXITSTATUS(os.system(runCommand))
        os.chdir(cwd)
        if self.Debugging:
            print "...Done: return value = %u." % rv
        return rv
    
    def runServer(self, test, subcommand):
        "Run a test server"
        testDirectory = self.testDirectory+"/"+test.Name()
        if self.Debugging:
            print "HardwareTestHarness.runServer:"
            print "    test directory %s" % (testDirectory)
        cwd=os.getcwd()
        os.chdir(testDirectory)
        runCommand = "make server %s=%s" % (
                                 Constants.RUNMODE, subcommand, 
                                 )
        print runCommand
        rv = os.WEXITSTATUS(os.system(runCommand))
        os.chdir(cwd)
        if self.Debugging:
            print "...Done: return value = %u." % rv
        return rv
    
    def saveSystemLog(self, test, run):
        "Save a section of the system log to the named file"
        # It feels a bit wrong to have this stuff hardcoded, but it works.
        marker_str=self.systemLogMarker+": %s: " % test.getName()
        syslog='/var/log/messages'
        log=open(syslog)
        contents = ""
        marker=False
        for l in log:
            if l.find(marker_str + "begin") >= 0:
                marker=True
            if marker:
                contents += l    
            if l.find(marker_str + "end") >= 0:
                break
        if marker == False:
            sys.stderr.write("Could not find log messages in %s!\n" % syslog)
        run.setSystemLog(contents)
            
    def saveOutput(self, run, retval, outputFilePath):
        "Append the md5sum-encoded return value to a log file"
        print "saveOutput: %s" % outputFilePath
        outputFile = open(outputFilePath)
        contents = ""
        for line in outputFile:
            contents += line
        contents += "\n" #make sure there's a line break at the end.
        # skip this  - do it later
        # m=md5.new()
        # m.update(contents)
        # sum=int(m.hexdigest(),16)
        # contents += (("%x\n" % (sum - retval)).upper())
        run.setReturnValue(retval)
        run.setTestOutput(contents)

   
    
    def setTestsEnabled(self):
        if self.Debugging:
            print "HardwareTest.setTestsEnabled:"
        if self.options.test:
            tests = self.certification.getTests(self.options.test)
        elif self.options.udi:
            tests = self.certification.getTestsByUDI(self.options.udi)
           
        disabled = True
        output = "Disabled"
        if self.options.enable:
            disabled = False
            output = "Enabled"
        for test in tests:
            test.setDisabled(disabled)
            if self.Debugging:
                print (output + " test: %s  udi: %s") % (test.getName(), test.getUDI())
        print (output + " %u tests") % len(tests)   
        
    def addTest(self):     
        if not self.options.udi:
            print "No device specified, use --udi <device udi> to specify device"
            return
        
        # otherwise
        hal=Hal()
        device = hal.getDeviceProperties(self.options.udi)
        if not device:
            print "No such device: %s " % self.options.udi
            return

        # otherwise     
        tests = list()   
        testInfo=TestInfo(self.testDirectory, self.Debugging)
        devices = list()
        devices.append(device)
        for test in testInfo.getTests(device):
            if self.Debugging:
                print "Checking test %s" % test.Name()
            if not self.options.test or (test.Name() == self.options.test):
                if self.Debugging:
                    print "added test: %s for device: %s (%s)" % (test, self.options.udi, device['info.product'])
                test = copy.copy(test)
                test.setPlanUDI(device['info.udi'])
                test.setPlanDeviceLabel(device['info.product'])
                tests.append(test)
                
        if len(tests) == 0 and self.options.test:
            test = testInfo.getTest(self.options.test)
            if not test:
                print "No such test %s" % self.options.test
                return
            # otherwise
            print "Warning, test %s does not match UDI %s" %(self.options.test, self.options.udi)
            test = copy.copy(test)
            test.setPlanUDI(device['info.udi'])
            test.setPlanDeviceLabel(device['info.product'])
            tests.append(test)
               
        addedTests = 0;
        for test in tests:
            self.certification.appendTest(name=test.Name(), udi=test.getPlanUDI(), label=test.getPlanDeviceLabel())
            addedTests = addedTests + 1
        
         
        self.certification.setPlanTime(self.getCurrentUTCTime()) 
        print "Added %u test(s)" % len(tests) 
         
    def getCurrentUTCTime(self):
        return time.gmtime(time.time())
            
