#!/usr/bin/python
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#
# The Report object packages and/or presents test results in text, rpm, or html form.
#
import os,sys,shutil, md5
import string

from hts.certificationtest import Certification, Test  
from hts.tags import Constants, Tags
from hts.controller import Controller

class Report(Controller):   
    
    def __init__(self, options, certification):
        self.certification = certification
        self.options = options
        self.Debugging = options.debug
        self.specFileName = "/usr/share/hts/lib/hts-results-rpm.spec"
        self.tarFileName = "hts-results.tar.bz2"
        
    def Write(self):
        if self.options.udi:
            tests = self.certification.getTestsByUDI(self.options.udi)
            self.WriteTests(tests)
        elif self.options.test:
            tests = self.certification.getTests(self.options.test, self.options.failures)
            self.WriteTests(tests)
        elif self.options.failures:
            tests = self.certification.getTests("", self.options.failures)
            self.WriteTests(tests)
        else:
            # no options - print the whole thing:
            # BTW - what about using XSLT to map the test into a document?
            print "\nRed Hat Hardware Certification test"
            print "--------------------------------------------"
            print "Test Suite:    %s    Release: %s" % (self.certification.getVersion(), self.certification.getRelease())
            print "Plan Created:  %s" % (self.certification.getPlanTime())
            print "--------------------------------------------"
            runNumber = 0
            for n in range(self.certification.getNumberOfTestRuns()):
                runNumber = n+1
                counts = self.certification.getTestCounts(runNumber)
                print "\nRun: %u on %s" % (runNumber, counts['start'])
                print "--------------------------------------------"              
                print "Tests: %u planned,  %u run, %u passed, %u failed" % ( counts['count'], counts['run'], counts['passed'], counts['failed'])
                print "--------------------------------------------\n"
                tests = self.certification.getTests() # all of them
                self.WriteTestRun(tests, runNumber)
                   
            if self.certification.getNumberOfTestRuns() > 0:
                counts = self.certification.getCombinedTestCounts()
                print "\nCombined Results for %s Runs:" % runNumber
                print "--------------------------------------------"
                print "   %u tests planned" % counts['count']
                print "   %u tests run" % counts['run']
                print "   %u tests always failed" % counts['neverPassed']
                print "   %u tests always passed" % counts['neverFailed']
            else:               
                self.WriteTestPlan()
                        
                
    def WriteTests(self, tests):        
        if self.options.latest:
            runNumber = self.certification.getNumberOfTestRuns()
            self.WriteTestRun(tests, runNumber)
        else: # all runs
            for n in range(self.certification.getNumberOfTestRuns()):
                runNumber = n+1
                self.WriteTestRun(tests, runNumber)
            
    def WriteTestRun(self, tests, runNumber):
        
        print "\nTest Run %s" % runNumber
        print "----------------------------------------------------------------"
        for test in tests:
            run = test.getRun(runNumber)
            if self.options.report == Constants.summary:
                if run:
                    returnValue = run.getReturnValue()
                    summary = "- %s" % run.getSummary()
                elif test.isDisabled():
                    returnValue = ""
                    summary = "- DISABLED"
                else:
                    returnValue = ""
                    summary = "- "
                 
                print "%-10s %-36s %s" % (
                        test.getName(),
                        test.getLabel(),
                        summary
                        )
                continue

            print "Test: %s: %s" % (test.getLabel(), test.getName())
            print "Tested UDI: %s" % test.getUDI()
            if run:
                print "Return Value: %s" % run.getReturnValue()
                print "Result: %s" % run.getSummary()  
                if self.options.report == Constants.full:
                    print "System Log:"
                    print run.getSystemLog()
                    print "Test Output:"
                    print run.getTestOutput()
            elif test.isDisabled():
                 print "DISABLED"
            else:
                print "not run"
                
            
                        
    def WriteTestPlan(self):
        print "\nTest Plan:"
        print "----------------------------------------------------------------"
        tests = self.certification.getTests()
        for test in tests:          
            disabled = ""
            if test.isDisabled():
               disabled = " - DISABLED"
             
            print "%-10s %-36s %s" % (test.getName(),  test.getLabel(),  disabled )
    
    def WriteResultsRPM(self, logDirectory):
        self.logDirectory = logDirectory
        varLogHTSDirectory = "/var/log/hts";
        # build the output directories under /var/log/hts/<vendor>-<model>-<os><release>_<arch>_results
        submissionDirectory = self.logDirectory + ("/%s_results" % (self.certification.getName()))
        print "writing results to %s" % submissionDirectory
        self.removeDirectory(submissionDirectory) # clean it, if it's there
        self.makeDirectoryPath(submissionDirectory)
        
        # write the test plan and results xml to etc/hts 
        etcDirectory = submissionDirectory + "/etc/hts"
        self.makeDirectoryPath(etcDirectory)
        shutil.copy(self.options.data + "/" + self.options.cfg, etcDirectory + "/.")
        shutil.copy(self.options.data + "/" + self.options.plan, etcDirectory + "/.")
        shutil.copy(self.options.data + "/" + self.options.results, etcDirectory + "/.")
        
        #write a system.conf for backwards compatability
        self.generateRHR2systemConf(etcDirectory)
        
        
        # write the test output to var/log/rhr
        for n in range(self.certification.getNumberOfTestRuns()):
            runNumber = n+1
            # write the test logs to [...]/var/log/rhr/runs/<runNumber-1>/<testname>
            # and  [...]/var/log/rhr/tests/<testname>/<runNumber-1>
            runDirectory = "%s%s/runs/%u" % (submissionDirectory, varLogHTSDirectory, runNumber-1)        
            self.removeDirectory(runDirectory)
            self.makeDirectoryPath(runDirectory)
            testsByName = self.certification.getTestsGroupedByName()
            for testName in testsByName.keys():
                if self.Debugging:
                    print "testName = " + testName
                test = testsByName[testName][0]
                # need upper case test name , to be compatible with HwCert Catalog            # YK
                byRunDirectory = runDirectory + ("/%s" % test.getName().upper())              # YK
                testDirectory = "%s%s/tests/%s" % (submissionDirectory, varLogHTSDirectory, test.getName().upper())   
                if runNumber == 1: self.removeDirectory(testDirectory)               
                byTestDirectory = testDirectory + ("/%u" % (runNumber-1))              
                self.WriteLogForRPM(testsByName[testName], runNumber, [byTestDirectory, byRunDirectory])
               
        # package the directories as an rpm
        os.system("tar -jcvf /usr/src/redhat/SOURCES/%s -C %s . > package.log 2>&1" % (self.tarFileName, submissionDirectory))
        self.generateResultsRPMSpec()
        os.system("rpmbuild -bb %s  >> package.log 2>&1" % self.specFileName)
        os.system("cp /usr/src/redhat/RPMS/noarch/hts-*.rpm " + self.options.data + "/.")
        
        # submit the results rpm
    
    def WriteLogForRPM(self, testsByName, runNumber, directories):
        for dir in directories:
            if self.Debugging:
                print "Writing logs to %s --------------------------" % dir
            self.removeDirectory(dir)
            deviceNumber = 0
            for test in testsByName:
                combinedTestPass = True
                
                if  not test.wasRun(runNumber):
                    continue
                    
                # otherwise
                if self.Debugging:
                    print "Writing results for test: %s  udi: %s  run: %u" % (test.getName(), test.getUDI(), runNumber) 
                run = test.getRun(runNumber)
                if not run.passed():
                    combinedTestPass = False
                if len(testsByName) > 1:
                    deviceNumber = deviceNumber + 1
                self.makeDirectoryPath(dir)
                outputLog = run.getTestOutput()
                if outputLog:
                    log = open(dir + "/output.log", "a")
                    log.write(outputLog)
                    log.close()
                systemLog = run.getSystemLog()
                if systemLog:
                    log = open(dir + "/system.log", "a")
                    log.write(systemLog)
                    log.close()
                halLog = run.getHalLog()
                if halLog:
                    log = open(dir + "/hal.log", "a")
                    log.write(halLog)
                    log.close()
                # it's ok if multiple devices overwrite this 
                self.generateHardwareLog(test, dir)
                
                self.copyAttachmentsToRPM(test, runNumber, deviceNumber, dir)       # YK
              #  self.copyAttachmentsToRPM(test, runNumber)                         # YK
               
            #add a new MD5 sum
            self.generateMD5Sum(dir, combinedTestPass)
                   
                        
                    
    def generateResultsRPMSpec(self):
        vendor = self.certification.getHardware(Tags.vendor)
        make = self.certification.getHardware(Tags.make)
        model = self.certification.getHardware(Tags.model)
        osname = self.certification.getOS(Tags.name)                # YangKun added this line
        osrelease = self.certification.getOS(Tags.release)          # YangKun added this line
        arch = self.certification.getHardware(Tags.arch)            # YangKun added this line
        # commment out now
        # htsRelease = self.certification.getHTSVersion()
        prefix = "/var/log/hts"
        packageName =  "hts-%s-%s" % (vendor, model)                
        packageName = packageName.replace(" ", "_")

        # to prevent the new result-rpm-package over-write the old one    # YK
        rpmRelease = "%(expr `ls /usr/src/redhat/RPMS/noarch/%{name}-%{version}-*[0-9].noarch.rpm | wc -l` + 1)" 
      #  rpmRelease = "1"   # YK
        tmpPath = "/tmp"
        
        
        specFile = open(self.specFileName, "w")
        specFile.writelines([
            "Summary: Red Hat hardware certification test results.\n",
            "Name: %s\n" % packageName,
            "Version: %s%s_%s_results\n" % (osname, osrelease, arch), 
            "Release: %s\n" % rpmRelease,
            "License: GPL\n",
            "Vendor: %s\n" % vendor,
            "Group: Applications/System\n",
            "Source: %s\n" % self.tarFileName, 
            "BuildRoot: %_tmppath/%name-%version-buildroot\n",
            "BuildArch: noarch\n",
            "Prefix: %s\n" % prefix,
            "\n",                            
            "%%description\n",
            "Red Hat Ready hardware certification test results.  Specific results for\n",
            "%s %s %s.\n" % (vendor, make, model),
            "\n",
            "%prep\n",
            "%setup -c rhr-results\n",
            "%build\n",
            "%install\n",
            "rm -rf $RPM_BUILD_ROOT\n",
            "mkdir -p $RPM_BUILD_ROOT/%{prefix}/%{name}-%{version}\n",
            "cp -a * $RPM_BUILD_ROOT/%{prefix}/%{name}-%{version}\n",
            "\n",
            "%clean\n",
            "rm -rf $RPM_BUILD_ROOT\n",
            "\n",
            "%files\n",
            "%defattr(-,root,root)\n",
            "%{prefix}/%{name}-%{version}\n",
            "\n",
            "%changelog\n",
            "* Tue Sep 12 2006 Greg Nichols <gnichols@redhat.com>\n",
            "- initial spec file based on rhr2 version\n",
            ])
        
    def generateRHR2systemConf(self, directory):       
        systemConfFile = open("%s/system.conf" % directory, "w")
        systemConfFile.writelines([
            "# RHR system.conf file\n",
            "# Generated by HTS for backwards compatability\n",
            "VENDOR=\"%s\"\n" % self.certification.getHardware(Tags.vendor),
            "MAKE=\"%s\"\n" % self.certification.getHardware(Tags.make),
            "MODEL=\"%s\"\n" % self.certification.getHardware(Tags.model),
            "CERTTYPE=\"%s\"\n" % "server", 
            "RHRRELEASE=\"%s\"\n" % self.certification.getHardware(Tags.model)
            ]) 
        systemConfFile.close()
       


    # We need hardware.log, not hardware.out 		# YK
    def generateHardwareLog(self, test, directory):
        hardwareCommand = "/usr/share/rhn/up2date_client/hardware.py"
        if not os.path.exists(hardwareCommand):
            print "Error: can not generate hardware.log: no command %s" % hardwareCommand
            return
        pipe = os.popen("python " + hardwareCommand)
        hardwareOutFile = open("%s/hardware.log" % directory, "w")
        # the HwCert Catalog have special requirements on some tests    # YK
        # TODO: need to make sure all tests get the right hwClass
        hwClassTmp = test.getName().upper()     # YK add
        if hwClassTmp == "CORE":                # YK add
             hwClass = "CPU"                    # YK add
        elif hwClassTmp == "INFO":              # YK add
             hwClass = "class"                  # YK add
        else:                                   # YK add
             hwClass = hwClassTmp               # YK add
        hwRecord = ""
        found = False
        while 1:
            line = pipe.readline()
            if line:
                 hwRecord += line
                 if len(line) < 2:
                     if found:
                         hardwareOutFile.write(hwRecord)
                         found = False
                     hwRecord = ""
                 elif line.find("class") > 0 and line.find(hwClass) > 0:
                         found = True
            else:
                 break
            
        pipe.close()
        hardwareOutFile.close()
       
   # def copyAttachmentsToRPM(self, test, runNumber):                      # YK
    def copyAttachmentsToRPM(self, test, runNumber, deviceNumber, dir):    # YK
        # look for attachments in test log output directories
        testOutputDir = self.logDirectory + ("/runs/%u/%s" % (runNumber, test.getName())) 
        try:
            for file in os.listdir(testOutputDir):
                if file != "output.log":
                    print "copied attachment file %s for %s test" % (file, test.getName())  
                    destinationDir = dir
                    if deviceNumber > 0:
                        destinationDir = "%s/device-%u" % (dir, deviceNumber)
                        self.makeDirectoryPath(destinationDir)
                    shutil.copy(testOutputDir+("/%s" % file), destinationDir)
        except OSError:
            print "Error: could not check test run log directory \"%s\" for test result attachments" % testOutputDir
                
        
    def generateMD5Sum(self, dir, combinedTestPass):
        if self.Debugging:
            print "combining test results md5 sum"
        try:
            log = open(dir + "/output.log", "a+")
            contents = ""
            returnValue = 1
            if combinedTestPass:
                returnValue = 0
            for line in log:
                contents += line
            m=md5.new()
            m.update(contents)
            sum=int(m.hexdigest(),16)
            # write the decimal format result to output.log, not hex       # YK
            log.write(("%u\n" % (sum - returnValue)).upper())              # YK
          #  log.write(("%x\n" % (sum - returnValue)).upper())             # YK
        except IOError:
            pass

        
        
