#!/bin/bash
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Ported by : YangKun <ykun@redhat.com>
#


######################################################################
######################################################################


# Functions

# Process arguments. Complain about unknown arguments, then ignore them. 
# The typical argument looks something like: hdc(cd-rw,dvd,burnfree)
process_dev(){
  unset cd_mode dvd_mode needs_eject specify_dev atapi burnfree
  # This takes all of the "(,)" characters in the argument and replaces them
  # with spaces, and then magically resets the positional parameters 
  # ($1, $2, ...) to each of the resulting words. e.g.:
  # hdc(cd-rw,dvd,burnfree) -> $1=hdc, $2=cd-rw, $3=dvd, $4=burnfree
  set -- ${1//[(,)]/ }
  # Grab the drive name
  cdrom_drive="$1"
  cdrecord_dev="/dev/$cdrom_drive"
  shift
  # and now process the rest of the arguments
  local item=""
  for item in $@; do
    case $item in
      cdrom|cd-r|cd-rw) cd_mode=$item ;;
      dvd|dvd-r|dvd-rw) dvd_mode=$item ;;
      eject) needs_eject="-eject" ;;
      burnfree) burnfree="driveropts=burnfree" ;;
      # translate-dev ? delete it ?	-YangKun
      translate-dev) cdrecord_dev="$(find_cdrecord_dev $cdrom_drive)" ;;
      dev=[0-9]_[0-9]_[0-9]) 
        specify_dev="$(echo $item | sed -e 's/_/,/g; s/raw=//')"
        cdrecord_dev="$atapi$specify_dev" ;;
      atapi)
        atapi="ATAPI:"
        if [ -n "$specify_dev" ]; then 
          cdrecord_dev="$atapi$specify_dev"
        fi
        ;;
      *) 
        echo "Warning: ignoring unknown CDROM argument \"$item\""
        ;; # Complain, but keep on going.
    esac
  done
  # This will cause the test to bail out if we don't have a test mode set.
  if [ -z "$cd_mode" ] && [ -z "$dvd_mode" ]; then
    echo "Error ! Invalid configuration !"
    return 1
  fi
}

# this function seems useless , delete it ?	-YangKun
# Attempt to translate given /dev into what cdrecord would like to see.
# Probably not that useful in 2.6 kernels.
find_cdrecord_dev(){
  local srdev=`echo $1 | sed -e 's/scd/sr/g'`
  local -a info=(`dmesg | grep "$srdev" | grep "Attached" | \
                  sed -e 's/scsi/scsi /g' -e 's/\,//g'`)
  local dev_string=${info[6]},${info[10]},${info[12]}
  echo ${dev_string[*]:-"0,0,0"}
}

# Try to close the tray on our own, otherwise get the user to do it.
handle_eject(){
  eject -t /dev/$cdrom_drive 2>/dev/null || {
  # clear
  echo "Please close tray or reload media in /dev/$cdrom_drive ."
  echo "And then press ENTER to continue !"
  read
  sleep 3
  }
}

# usage: cmp_tree dir1 dir2
# compares regular files in dir1 and dir2; returns success (0) if all match.
cmp_tree(){
  ( cd $1; find . -type f ) | while read file; do
     cmp "$1/$file" "$2/$file" || exit 1 # exit from the pipe subshell
  done
}

# return success if the drive supports TAO recording.
supports_tao(){ 
  cdrecord dev=$1 -checkdrive 2>&1 | grep -q 'modes.*TAO'
}

# wrapper function that figures out the proper arguments for cdrecord
# and does some prep stuff for us.
# NOTE: uses $cdrecord_dev, $needs_eject, and $burnfree (set by process_dev)
do_cdrecord(){
  local writemode=""
  # Unmount the drive before we start messing with it
  grep -q $cdrecord_dev /proc/mounts && umount $cdrecord_dev
  # Use SAO if the device doesn't support TAO
  if ! supports_tao $cdrecord_dev ; then
    writemode="-sao"
  fi
  # write the disk, if write succeeded, return 0	-YangKun
  if cdrecord -v $writemode $needs_eject $burnfree $@ ; then
    echo "do_cdrecord Write disk succeeded !"
  else
    echo "do_cdrecord Error ! Write disk failed !"
    return 1
  fi
}

# return success if the drive supports burnfree.
supports_burnfree(){
  cdrecord dev=$1 -checkdrive 2>&1 | grep -q 'flags.*BURNFREE'
}

# configure the cdrom(s)			-YangKun
# return cdrom names and supported features	-YangKun
config_cdrom(){
  local procfile="/proc/sys/dev/cdrom/info"
  local device="$1"
  local position=2 # offset to account for proc titles
  for dev in `head -n3 $procfile | tail -n 1 | sed -e 's/sr/scd/g'`
  do
    [ $dev = $device ] && break
    let ++position
  done
  # ensure we don't capture previous settings
  local features="cdrom"
  # This looks for lines like: Can write DVD-R:  0   1
  # and transforms it to:                dvd_r=1
  # which is then evaluated by the shell, thus setting the variables used below.
  eval $(awk "/Can (read|write) (CD|DVD)/ {print \$3 \$$position}" $procfile | \
         tr A-Z:- a-z=_)
  # Each CD type is a superset of the previous ones.
  [ "$cd_r" == 1 ]    && features="cd-r"
  [ "$cd_rw" == 1 ]   && features="cd-rw"
  # Handle DVD capabilities.
  if [ "$dvd_r" == 1 ]; then
    # NOTE: While early DVD-R drives couldn't write DVD-RW media, nearly all
    # modern drives can, so we assume that DVD-R implies DVD-RW. You can
    # override this by changing "dvd-rw" to "dvd-r" in hardware.conf.
#    features="$features,dvd-rw"
# do we have to check "dvd-rw" capability for a dvd-r drive ? make things more dirtier ?	-YangKun
    features="$features,dvd-r"
  elif [ "$dvd" == 1 ] ; then
    features="$features,dvd"
  # We have no DVD-RAM test - I don't know if we officially support it anyway.
  #elif [ "$dvdram == 1" ]; then features="$features,dvd-ram"
  fi
  local cdrecord_dev="/dev/$device"
  # If the device supports burnfree, add it to the list
  umount /dev/$device umount 2>/dev/null
  if supports_burnfree $cdrecord_dev; then
    features="$features,burnfree"
  fi
  # return with features attached
  echo "$device($features)"
}

# to get the name of cd-rom devices. 	-YangKun
# GBN - the harness does this - commented out here
# get_cdrom_info() {
#  for m in `hal-device | grep block.device | awk '{print $3}' | tr -d "'" | colrm 1 5 | sort -u `
#  do
#    if grep -q "$m" /proc/sys/dev/cdrom/info ; then
#      echo $m
#    fi
#  done
#}

# get-device-info <UDI> <HAL Attribute Name>
# returns the HAL attribute value for a device
# 
get-device-info(){
    hal-device  $1 | fgrep $2 | awk '{print $3}' | sed -e "s/[']//g" 
}

# GBN: we want the test to run only for the UDI passed by 
# the test harness.  get the device w/o the /dev/ prefix.
CDROM_DEVICES=`get-device-info $UDI block.device | sed -e "s/\/dev\///g"`


# echo DVD info if $dev_mode is set     -YangKun
echo_dvd() {
  if [ $dvd_mode == dvd ]   ; then echo "   --- one DVD data disk(e.g. one RHEL5 install DVD disk)";fi
  if [ $dvd_mode == dvd-r ] ; then echo "   --- one blank DVD-R disk"; fi
  if [ $dvd_mode == dvd-rw ]; then echo "   --- one good DVD-RW disk"; fi
}


######################################################################
######################################################################


# reader
# handles CD and DVD ROM drives
reader(){
  mkdir -p ./tmp ./device
  mount /dev/$cdrom_drive ./device 2>/dev/null
  if [ -b /dev/$cdrom_drive ]; then
    local media_size=`df -k -P /dev/$cdrom_drive | tail -n1 | awk ' { print $3; } '`
  else
    # otherwise it's expected to be a file
    local media_size=`ls -sk $cdrom_drive | awk ' { print $1; } '`
  fi
  # check the inserted media is CD or DVD 	-YangKun
  # we'll take a guess that anything bigger than 800MB is a DVD
  case $1 in
    dvd) if [ $media_size -lt 800000 ]; then 
           echo "$cdrom_drive appears NOT to be a DVD !"
           return 1
         fi;;
    cdrom) if [ $media_size -gt 800000 ]; then
             echo "$cdrom_drive appears to be a DVD !"
             return 1
           fi;;
  esac
  rm -rf ./device-copy
  cp -a ./device ./device-copy
  echo "Start to compare data, this may take a long time ...."
  cmp_tree ./device ./device-copy
  umount ./device
}

test_cdrom(){
  # ask user to insert a disk 		-YangKun
  eject /dev/$cdrom_drive 2>/dev/null
  # clear
  echo "Please insert a $cd_mode data disk into $cdrom_drive !"
  echo "And then press ENTER to continue !"
  read
  sleep 3
  echo "Start testing CD-ROM ...."
  sleep 1
  reader cdrom
}

test_dvd(){
  # ask user to insert a disk 		-YangKun
  eject /dev/$cdrom_drive 2>/dev/null
  # clear
  echo "Please insert a $dvd_mode data disk into $cdrom_drive !"
  echo "And then press ENTER to continue !"
  read
  sleep 3
  echo "Start testing DVD-ROM ...."
  sleep 1
  reader dvd
}


######################################################################
######################################################################


# writer
# Handle CD and DVD burners
# We test by writing a chunk of the filesystem to the media and then checking the file sums
writer(){
  case $1 in
    cdrom) disk_mode=$cd_mode
           local iso_dir=/usr/bin;;   # ranges from 100MB-270MB
    dvd)   disk_mode=$dvd_mode
           local iso_dir=/usr/share;; # ranges from 970MB-3.5GB
  esac
  cdblocks=$(mkisofs -quiet -R -print-size $iso_dir)
  # Okay, write the directory to disk
  mkisofs -quiet -R $iso_dir | do_cdrecord dev=$cdrecord_dev fs=32M tsize=${cdblocks}s -
  ${needs_eject:+`handle_eject`}
  mkdir -p media
  mount /dev/$cdrom_drive media 2>/dev/null
  echo "Start to compare data, this may take quite a long time, please be patient ...."
  cmp_tree $iso_dir media
  umount media
}

test_cd-r(){
  # ask user to insert a disk 		-YangKun
  eject /dev/$cdrom_drive 2>/dev/null
  # clear
  echo "Please insert a blank $cd_mode disk into $cdrom_drive !"
  echo "And then press ENTER to continue !"
  read
  sleep 3
  echo "Start testing CD-R ...."
  sleep 1
  writer cdrom
}

test_dvd-r(){
  # ask user to insert a disk 		-YangKun
  eject /dev/$cdrom_drive 2>/dev/null
  # clear
  echo "Please insert a blank $dvd_mode disk into $cdrom_drive !"
  echo "And then press ENTER to continue !"
  read
  sleep 3
  echo "Start testing DVD-R ...."
  sleep 1
  writer dvd
}


######################################################################
######################################################################


# re-writer
# Handled CD and DVD RW burners
# ...blank being the difference...
rewriter(){
  # Blank the disk.
  do_cdrecord dev=$cdrecord_dev blank=fast || { 
    echo "Error ! The disk you've inserted is bad media or is not re-writable !"
    return 1
  }
  ${needs_eject:+`handle_eject`}
  # rewrite now	!	-YangKun
  writer $1
}

test_cd-rw(){
  # ask user to insert a disk 		-YangKun
  eject /dev/$cdrom_drive 2>/dev/null
  # clear
  echo "Please insert a good $cd_mode disk into $cdrom_drive !"
  echo "And then press ENTER to continue !"
  read
  sleep 3
  echo "Start testing CD-RW ...."
  sleep 1
  rewriter cdrom
}

test_dvd-rw(){
  # ask user to insert a disk 		-YangKun
  eject /dev/$cdrom_drive 2>/dev/null
  # clear
  echo "Please insert a good $dvd_mode disk into $cdrom_drive !"
  echo "And then press ENTER to continue !"
  read
  sleep 3
  echo "Start testing DVD-RW ...."
  sleep 1
  rewriter dvd
}


######################################################################
######################################################################


# man function		-YangKun
cdrom_test(){
  for dev in $CDROM_DEVICES ; do
    new_dev=`config_cdrom $dev`
    features=`echo $new_dev | colrm 1 3`
    process_dev $new_dev
    # show what features we found		-YangKun
    # clear
    echo "Your CD/DVD-ROM device /dev/$dev supports $features "
    echo "We will check all these features. "
    echo "You need to prepare following media(disks) for this test:"
    echo " "
    if [ $cd_mode == cdrom ]  ; then echo "   --- one CD data disk(e.g. one RHEL5 install CD disk)"; fi
    if [ $cd_mode == cd-r ]   ; then echo "   --- one blank CD-R disk"; fi
    if [ $cd_mode == cd-rw ]  ; then echo "   --- one good CD-RW disk"; fi
    ${dvd_mode:+echo_dvd}
    echo " "
    echo "Press ENTER when you are ready !"
    echo "Or press Ctrl-C to terminate this test !"
    read
    sleep 1
    # handles CD-related features 		-YangKun
    if ${cd_mode:+test_$cd_mode}; then
      echo "test_$cd_mode succeeded !"
    else
      echo "Error ! test_$cd_mode failed !"
      return 1
    fi
    # handles DVD-related features		-YangKun
    if ${dvd_mode:+test_$dvd_mode}; then
      echo "test_$dvd_mode succeeded !"
    else
      echo "Error ! test_$dvd_mode failed !"
      return 1
    fi
    # remove temp files		-YangKun
    rm -rf tmp device media
   done
}

# run the test		-YangKun
cdrom_test

# TODO:		-YangKun
# need to check on IBM s390/zSeries


