#!/bin/bash
# Copyright (c) 2006 Red Hat, Inc. All rights reserved. This copyrighted material
# is made available to anyone wishing to use, modify, copy, or
# redistribute it subject to the terms and conditions of the GNU General
# Public License v.2.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Ported by : YangKun <ykun@redhat.com>
#

# Min and max blocksizes, in bytes. For each test, we loop through block sizes
# starting with $MIN_BS and doubling until we hit $MAX_BS. Both should be
# powers of 2 in the range [1024-65536].
DEFAULT_MIN_BS=1024
DEFAULT_MAX_BS=65536
# temprarily changed to a small value for fast running   -YangKun
#DEFAULT_MAX_BS=2048

# Maximum size of the test area, in KB. (defaults to 1GB)
MAX_SIZE=1048576
# temprarily changed to a small value for fast running   -YangKun
#MAX_SIZE=2048

# The pvs binary, for LVM support.
pvs="/usr/sbin/pvs"


# Helper function: run dt with various block sizes and reasonable defaults
do_dt() {
    for ((bs=$MIN_BS; $bs<=$MAX_BS; bs*=2)); do
        dt enable=aio aios=8 pattern=0xDEADBEEF bs=$bs $@
    done
}


# usage: dt_test FILE_OR_DEVICE SIZE_IN_KB
# Do a few passes of read/write testing on the given device.
dt_test() {
    local file=$1; shift
    local size=$1; shift
    # Restrict size to $MAX_SIZE
    if [ $size -gt $MAX_SIZE ]; then size=$MAX_SIZE; fi
    # Find out how many $MAX_BS-sized blocks we can fit on the test file/dev
    local blocks=$(($size/($MAX_BS/1024))) 
    echo "--- sequential write"
    do_dt of=$file records=$blocks iotype=sequential dispose=keep disable=verify $@
    echo "--- sequential read"
    do_dt if=$file records=$blocks iotype=sequential $@
    local seed=$RANDOM # We need to use the same seed for both random passes
    if [ ${file:0:12} == "/dev/raw/raw" ];then limit=rlimit;else limit=dlimit;fi
    echo "--- random write"
    do_dt of=$file records=$blocks iotype=random dispose=keep disable=verify \
          ralign=$MIN_BS rseed=$seed $limit=$MAX_SIZE $@
    echo "--- random read"
    do_dt if=$file records=$blocks iotype=random \
          ralign=$MIN_BS rseed=$seed $limit=$MAX_SIZE $@
}


# usage: mkrawdev BLOCKDEV
# Attaches a /dev/raw device to BLOCKDEV, and prints its name.
mkrawdev() {
    local rawdev=1 waited=0
    # find an unused raw device
    while raw -qa | grep -q "^/dev/raw/raw$rawdev"; do let rawdev++; done
    rawdev="/dev/raw/raw$rawdev"
    raw $rawdev /dev/$testdev &>/dev/null # Bind to disk device
    # Wait for udev/devfs to create the device file
    while [ $waited -lt 5 ] && [ ! -e $rawdev ]; do 
        sleep 1; let waited++
    done
    # Okay, find out if it really got created
    if [ -e $rawdev ]; then
        echo $rawdev
        return 0
    else
        echo "Can't find raw device $rawdev!" > /dev/stderr
        return 1
    fi
}


# usage: devsize [block device file]
# Find the size of the given device.
devsize() {
    local maj=0 min=0
    [ -b $1 ] || return 1
    # We use 'file' to find the major/minor numbers for the device, then sed
    # to change "/dev/hda1: block special (3/1)" into "maj=3 min=1"
    eval $(file -L $1|sed -r 's|.*special .([0-9]+)/([0-9]+).|maj=\1 min=\2|')
    # Now look up the device by maj/min number in /proc/partitions.
    local size=$(grep "^ *$maj  *$min " /proc/partitions | awk '{print $3}')
    [ -n "$size" ] || return 1
    echo $size
}


# usage: is_lvm_dev_on_device /dev/mapper/VolGroup00-LogVol01
# Return true if the given device is actually an LV, and that LV is only on
# the device we're trying to test.
is_lvm_dev_on_device() {
    local devnum=$(ls -l $1 | tr -d , | awk '{print $5"  "$6}')
    local lvs="/usr/sbin/lvs -okernel_major,kernel_minor,devices"
    $lvs | grep "$devnum" | grep -q $device && 
        [ $($lvs | grep "$devnum" | wc -l) == 1 ]
}


# Unmount, if the device is mounted
do_umount() { if grep -q "/dev/$1\\b" /proc/mounts; then umount /dev/$1; fi; }


# In case of emergency..
error_handler() { 
    do_umount $testdev;
    if [ "$testdev" == "$swapdev" ]; then restore_swap $swapdev; fi
}


# restore a swap device, given its short name (e.g. 'hda')
restore_swap() {
    if ! grep -q "^/dev/$1" /proc/swaps; then
        echo "restoring swap device /dev/$1"
        mkswap /dev/$1; swapon /dev/$1
    fi
}


# find the storage devices
# we want the test to run only for the UDI passed by the test harness.
get_storage_info() {
     hal-get-property --udi $UDI --key block.device |  colrm 1 5 | sort -u
}
STORAGE_DEVICES=`get_storage_info`

# Overall test strategy: Instruct the tester to install RHEL *only* on
# a single disk. We can use the swap partition for testing that disk, and
# every other (unused) disk we can test directly.
storage_test() {
#    # check whether running on RHEL5	-YangKun
#    RHEL5=`grep "Tikanga" /etc/redhat-release`
##    RHEL5=`grep "Bordeaux" /etc/redhat-release`
#    if [ -z $RHEL5 ]; then
#      echo "Error ! This test only supports RHEL5 ! You can not run it on Fedora Core 5 and other OSs !"
#      return 1
#    fi

# check whether dt package is installed 	-YangKun
    if ! which dt 2>/dev/null; then
      echo "Error ! No dt pachage found !"
      return 1
    else
      echo "Find dt package, continue testing !"
    fi 


    for device in $STORAGE_DEVICES; do
      local swapdev="" testdev="" size=0 is_lvm=false
      # Device sanity check: does it exist?
      if grep -q "$device\b" /proc/partitions; then
        echo "find $device in /proc/partitions ! start testing on $device !"
      
        # If there's a swap device, use that for testing
        swapon -a # turn 'em all on first, just in case
        swapdev=$(sort -rgk3 /proc/swaps | grep -m1 "^/dev/$device" || true)
        # RHEL4 makes swap on LVM; check to see if this is what we have
        # need to check whether RHEL5 plays the same way	-YangKun
        if [ -z "$swapdev" ] && [ -x $pvs ]; then
            # if we find a swap device that's an LVM LV, and that LV
            # is restricted to the target device, then we can use that
            for s in $(sort -rgk3 /proc/swaps | awk '/^\/dev/ {print $1}'); do
                if is_lvm_dev_on_device $s; then 
                    swapdev=$s;
                    is_lvm=true;
                    break;
                fi
            done
        fi
        if [ -n "$swapdev" ]; then
            # Found a swap device! Whew! Cut it down to the short name.
            swapdev=${swapdev%% *}; swapdev=${swapdev#/dev/}
            #
            # added for debugging	-YangKun
            echo "-----------------swapdev is $swapdev --------------------"
            echo "-----------------testdev is $testdev --------------------"
            echo "-----------------'is_lvm' ? $is_lvm  --------------------"
            #
            echo "Using swap device $swapdev for testing."
            swapoff /dev/$swapdev # Turn off swapping. 
            testdev=$swapdev
        else 
            # No swap, but if the disk is unused, we'll test with it.
            # Is it in use by LVM?
            if [ -x $pvs ] && $pvs | grep -q "^ */dev/$device"; then
                echo "Error: $device is currently in use by LVM."
                echo "It cannot be tested. You may need to reinstall."
                return 1
            fi
            # Is it part of a RAID set?
            if grep -q "active.*$device" /proc/mdstat; then
                echo "Error: $device is part of a RAID set."
                echo "It cannot be tested. You may need to reinstall."
                return 1
            fi
            # Is there an active filesystem on it?
            if grep -q "^/dev/$device" /proc/mounts /etc/mtab; then
                echo "Error: $device has active filesystems on it."
                echo "It cannot be tested. You may need to reinstall."
                return 1
            fi
            # No? Awesome! We can use this device for testing.
            testdev=$device
        fi
        # TODO: 'destructive test' warnings should maybe go here?

        # Set device-appropriate blocksizes and other special stuff.
        # for IBM s390/zSeries ?                   -YangKun
        case $device in
            dasd*) MIN_BS=4096; MAX_BS=$DEFAULT_MAX_BS
              # You can't use a dasd without a partition table..
              if [ ${testdev:0:4} == dasd ] && [ ${testdev%%[0-9]} == $testdev ]; then
                  fdasd -a /dev/$testdev
                  testdev=${testdev}1
              fi
            ;;
            *)     MIN_BS=$DEFAULT_MIN_BS; MAX_BS=$DEFAULT_MAX_BS ;;
        esac
        
        # If anything goes wrong, restore the device to a sane state.
        trap error_handler EXIT ERR INT TERM
        
        # VFS (buffered filesystem) testing.
        # Create a new filesystem and mount it.
        local mountdir=$(mktemp -d $(echo $device|tr / .).XXXXXX)
        mkfs -t ext3 -F /dev/$testdev
        mount -t ext3 /dev/$testdev $mountdir
        echo "*** Testing buffered filesystem (ext3) performance on $device"
        size=$(df -Pk $mountdir | awk '{print $4}' | tail -n1)
        dt_test $mountdir/dt-testfile $size
        do_umount $testdev

        echo "*** Testing raw I/O performance on $device"
        size=$(devsize /dev/$testdev)

        # RHEL5's kernel is 2.6.*, use O_DIRECT      -YangKun
        dt_test /dev/$testdev $size flags=direct
        
        #
        # added for debugging	-YangKun
        echo "-----------------swapdev is $swapdev --------------------"
        echo "-----------------testdev is $testdev --------------------"
        echo "-----------------'is_lvm' ? $is_lvm  --------------------"
        #
        # Clean up: turn off error handler, restore swap if needed.
        trap - EXIT ERR INT TERM
        if [ "$testdev" == "$swapdev" ]; then restore_swap $swapdev; fi

        # remove temp directories	-YangKun
        #rmdir $device.* 2>/dev/null

      fi
    done
}

# run the test !      -YangKun
storage_test

# TODO: need to check on IBM s390/zSeries 	-YangKun


